import { evalToBool } from '@anjuna/core';
import { CommonModule } from '@angular/common';
import { FormGroup, FormControl, Validators, FormArray, FormsModule, ReactiveFormsModule } from '@angular/forms';
import { AnjunaValidators, BaseTranslationPipe, AnjunaCoreModule } from '@anjuna/angular-core';
import { HostBinding, Input, Component, ChangeDetectionStrategy, ViewEncapsulation, Renderer2, ElementRef, HostListener, Output, EventEmitter, Injectable, Directive, TemplateRef, Pipe, ChangeDetectorRef, CUSTOM_ELEMENTS_SCHEMA, NgModule, ContentChild, ViewChild, defineInjectable } from '@angular/core';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/* @internal */
/**
 * @abstract
 */
class BaseSchema {
    /**
     * @return {?}
     */
    get hideLabels() {
        return this._hideLabels;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set hideLabels(value) {
        if (typeof value === 'string') {
            this._hideLabels = evalToBool(value);
        }
        else {
            this._hideLabels = value;
        }
    }
    /**
     * @param {?} index
     * @return {?}
     */
    trackByFn(index) {
        return index;
    }
}
BaseSchema.propDecorators = {
    hideLabels: [{ type: Input }],
    unitOfMeasurement: [{ type: Input }],
    _hideLabels: [{ type: HostBinding, args: ['attr.hideLabel',] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ParsedField {
    /**
     * @param {?} name
     * @param {?} fieldDef
     * @param {?=} requiredFields
     * @param {?=} readonlyFields
     */
    constructor(name, fieldDef, requiredFields = [], readonlyFields = []) {
        this.default = fieldDef.default;
        this.description = fieldDef.description;
        this.enforceUnique = fieldDef.uniqueItems;
        this.format = fieldDef.format;
        this.items = fieldDef.items;
        this.label = fieldDef.title || fieldDef.label || name;
        this.maximum = fieldDef.maximum;
        this.maxItems = fieldDef.maxItems;
        this.maxLength = fieldDef.maxLength;
        this.minimum = fieldDef.minimum;
        this.minItems = fieldDef.minItems;
        this.minLength = fieldDef.minLength;
        this.name = name;
        this.nestedDef = fieldDef;
        this.options = fieldDef.enum;
        this.pattern = fieldDef.pattern;
        this.parsedArray = fieldDef.parsedArray;
        this.parsedList = fieldDef.parsedList;
        this.parsedFields = fieldDef.parsedFields;
        this.properties = fieldDef.properties;
        this.readonly = fieldDef.readonly || (readonlyFields ? readonlyFields.includes(name) : false);
        this.required = requiredFields ? requiredFields.includes(name) : false;
        this.subtype = fieldDef.subtype;
        this.type = fieldDef.type;
    }
    /**
     * @return {?}
     */
    toInputType() {
        if (this.format === 'date') {
            return 'date';
        }
        else if (this.options) {
            return 'select';
        }
        else if (this.type === 'integer' || this.type === 'number') {
            return 'number';
        }
        else if (this.type === 'boolean') {
            return 'checkbox';
        }
        else if (this.type === 'array' && this.subtype === 'tier') {
            return 'tier';
        }
        else if (this.type === 'array' && this.items && this.items.type === 'object') {
            return 'table';
        }
        else if (this.type === 'array' && this.items) {
            return 'list';
        }
        else if (this.type === 'object' && this.properties) {
            return 'nested';
        }
        else if (this.type === 'string' && this.subtype === 'area') {
            return 'area';
        }
        return 'text';
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/* @internal */
/**
 * @abstract
 */
class SchemaField extends BaseSchema {
    constructor() {
        super();
        this.isField = true;
    }
}
SchemaField.propDecorators = {
    field: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class SchemaFieldComponent extends SchemaField {
    /**
     * @param {?} element
     * @param {?} renderer
     */
    constructor(element, renderer) {
        super();
        this.element = element;
        this.renderer = renderer;
        this.tierToChange = new EventEmitter();
    }
    /**
     * @param {?} event
     * @return {?}
     */
    _onFieldChange(event) {
        if (this.field.name === 'tierTo') {
            this.tierToChange.emit(this.index);
        }
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        this.renderer.addClass(this.element.nativeElement, this.field.name.split(' ').join('_'));
    }
}
SchemaFieldComponent.decorators = [
    { type: Component, args: [{
                selector: 'anj-schema-field',
                template: "<ng-container [ngSwitch]=\"field.toInputType()\">\n    <ng-template ngSwitchCase=\"date\">\n        <anj-form-group>\n            <anj-label *ngIf=\"!hideLabels\">{{field.label}}</anj-label>\n            <anj-input [formControl]=\"control\" placeholder=\"{{hideLabels ? field.name : null}}\" [min]=\"field.minimum\" [max]=\"field.maximum\" [attr.readonly]=\"field.readonly\">\n                <anj-date-picker-toggle></anj-date-picker-toggle>\n                <anj-date-picker></anj-date-picker>\n            </anj-input>\n            <ng-template [ngIf]=\"control.errors\">\n                <anj-form-error *ngIf=\"control.errors?.required\">\n                    {{field.label || field.name}} {{'is required.' | anjSchemaTranslate}}\n                </anj-form-error>\n                <anj-form-error *ngIf=\"control.errors?.date?.valid === false\">\n                    {{field.label || field.name}} {{'is not a valid date.' | anjSchemaTranslate}}\n                </anj-form-error>\n            </ng-template>\n            <anj-form-helper *ngIf=\"field.description\">{{field.description}}</anj-form-helper>\n        </anj-form-group>\n    </ng-template>\n    <ng-template ngSwitchCase=\"select\">\n        <anj-form-group>\n            <anj-label *ngIf=\"!hideLabels\">{{field.label}}</anj-label>\n            <anj-select placeholder=\"{{hideLabels ? field.name : null}}\" [formControl]=\"control\">\n                <anj-option *ngFor=\"let option of field.options\" [value]=\"option\">{{option}}</anj-option>\n                <ng-template [ngIf]=\"control.errors\">\n                    <anj-form-error>{{'Please select an option.' | anjSchemaTranslate}}</anj-form-error>\n                </ng-template>\n                <anj-form-helper *ngIf=\"field.description\">{{field.description}}</anj-form-helper>\n            </anj-select>\n        </anj-form-group>\n    </ng-template>\n    <ng-template ngSwitchCase=\"number\">\n        <anj-form-group>\n            <anj-label *ngIf=\"!hideLabels\">{{field.label}}</anj-label>\n            <anj-input [formControl]=\"control\" step=\"1\" type=\"number\" placeholder=\"{{hideLabels ? field.name : null}}\" [min]=\"field.minimum\" [max]=\"field.maximum\" [attr.readonly]=\"field.readonly\">\n                <anj-number-stepper></anj-number-stepper>\n            </anj-input>\n            <ng-template [ngIf]=\"control.errors\">\n                <anj-form-error *ngIf=\"control.errors?.required\">\n                    {{field.label || field.name}} {{'is required.' | anjSchemaTranslate}}\n                </anj-form-error>\n                <anj-form-error *ngIf=\"control.errors?.min\">\n                    {{field.label || field.name}} {{'must be at least' | anjSchemaTranslate}} {{control.errors.min.min}}.\n                </anj-form-error>\n                <anj-form-error *ngIf=\"control.errors?.max\">\n                    {{field.label || field.name}} {{'must not be greater than' | anjSchemaTranslate}} {{control.errors.max.max}}.\n                </anj-form-error>\n            </ng-template>\n            <anj-form-helper *ngIf=\"field.description\">{{field.description}}</anj-form-helper>\n        </anj-form-group>\n    </ng-template>\n    <ng-template ngSwitchCase=\"checkbox\">\n        <anj-check-group>\n            <anj-checkbox [formControl]=\"control\">{{field.label}}</anj-checkbox>\n            <ng-template [ngIf]=\"control.errors\">\n                <anj-form-error *ngIf=\"control.errors?.required\">\n                    {{field.label || field.name}} {{'is required.' | anjSchemaTranslate}}\n                </anj-form-error>\n            </ng-template>\n        </anj-check-group>\n    </ng-template>\n    <ng-template ngSwitchCase=\"list\">\n        <anj-schema-list [field]=\"field\" [control]=\"control\" [label]=\"!hideLabels ? field.label: null\"></anj-schema-list>\n        <ng-template [ngIf]=\"control.errors\">\n            <anj-form-error *ngIf=\"control.errors?.unique\">\n                {{field.label || field.name}} {{'must not have duplicate values.' | anjSchemaTranslate}}\n            </anj-form-error>\n            <anj-form-error *ngIf=\"control.errors?.minlength\">\n                {{field.label || field.name}} {{'must have at least' | anjSchemaTranslate}} {{control.errors['minlength'].requiredLength}} {{'entries.' | anjSchemaTranslate}}\n            </anj-form-error>\n        </ng-template>\n    </ng-template>\n    <ng-template ngSwitchCase=\"nested\">\n        <anj-schema-nested-field [field]=\"field\" [control]=\"control\" [label]=\"!hideLabels ? field.label: null\"></anj-schema-nested-field>\n    </ng-template>\n    <ng-template ngSwitchCase=\"table\">\n        <anj-schema-table [field]=\"field\" [control]=\"control\" [label]=\"!hideLabels ? field.label: null\"></anj-schema-table>\n    </ng-template>\n    <ng-template ngSwitchCase=\"tier\">\n        <anj-schema-tier [field]=\"field\" [control]=\"control\" [unitOfMeasurement]=\"unitOfMeasurement\" [label]=\"!hideLabels ? field.label: null\"></anj-schema-tier>\n    </ng-template>\n    <ng-template ngSwitchCase=\"area\">\n        <anj-form-group>\n            <anj-label *ngIf=\"!hideLabels\">{{field.label}}</anj-label>\n            <anj-textarea [formControl]=\"control\" placeholder=\"{{hideLabels ? field.name : null}}\" [attr.readonly]=\"field.readonly\"></anj-textarea>\n            <ng-template [ngIf]=\"control.errors\">\n                <anj-form-error *ngIf=\"control.errors.required\">\n                    {{field.label || field.name}} {{'is required.' | anjSchemaTranslate}}\n                </anj-form-error>\n                <anj-form-error *ngIf=\"control.errors.minlength\">\n                    {{field.label || field.name}} {{'must be at least' | anjSchemaTranslate}} {{control.errors.minlength.requiredLength}} {{'characters.' | anjSchemaTranslate}}\n                </anj-form-error>\n                <anj-form-error *ngIf=\"control.errors.maxlength\">\n                    {{field.label || field.name}} {{'must not be greater than' | anjSchemaTranslate}} {{control.errors.maxlength.requiredLength}} {{'characters.' | anjSchemaTranslate}}\n                </anj-form-error>\n                <anj-form-error *ngIf=\"control.errors.pattern\">\n                    {{field.label || field.name}} {{'must adhere to a specific pattern' | anjSchemaTranslate}}{{field.description ? ': ' + field.description : null}}\n                </anj-form-error>\n            </ng-template>\n            <anj-form-helper *ngIf=\"field.description\">{{field.description}}</anj-form-helper>\n        </anj-form-group>\n    </ng-template>\n    <ng-template ngSwitchDefault>\n        <anj-form-group>\n            <anj-label *ngIf=\"!hideLabels\">{{field.label}}</anj-label>\n            <anj-input [formControl]=\"control\" placeholder=\"{{hideLabels ? field.name : null}}\" [attr.readonly]=\"field.readonly\"></anj-input>\n            <ng-template [ngIf]=\"control.errors\">\n                <anj-form-error *ngIf=\"control.errors.required\">\n                    {{field.label || field.name}} {{'is required.' | anjSchemaTranslate}}\n                </anj-form-error>\n                <anj-form-error *ngIf=\"control.errors.minlength\">\n                    {{field.label || field.name}} {{'must be at least' | anjSchemaTranslate}} {{control.errors.minlength.requiredLength}} {{'characters.' | anjSchemaTranslate}}\n                </anj-form-error>\n                <anj-form-error *ngIf=\"control.errors.maxlength\">\n                    {{field.label || field.name}} {{'must not be greater than' | anjSchemaTranslate}} {{control.errors.maxlength.requiredLength}} {{'characters.' | anjSchemaTranslate}}\n                </anj-form-error>\n                <anj-form-error *ngIf=\"control.errors.pattern\">\n                    {{field.label || field.name}} {{'must adhere to a specific pattern' | anjSchemaTranslate}}{{field.description ? ': ' + field.description : null}}\n                </anj-form-error>\n                <anj-form-error *ngIf=\"control.errors.email\">\n                    {{field.label || field.name}} {{'must adhere to an email pattern (ex. name@domain.com)' | anjSchemaTranslate}}\n                </anj-form-error>\n            </ng-template>\n            <anj-form-helper *ngIf=\"field.description\">{{field.description}}</anj-form-helper>\n        </anj-form-group>\n    </ng-template>\n</ng-container>\n",
                changeDetection: ChangeDetectionStrategy.OnPush,
                encapsulation: ViewEncapsulation.None
            }] }
];
/** @nocollapse */
SchemaFieldComponent.ctorParameters = () => [
    { type: ElementRef },
    { type: Renderer2 }
];
SchemaFieldComponent.propDecorators = {
    control: [{ type: Input }],
    index: [{ type: Input }],
    tierToChange: [{ type: Output }],
    isField: [{ type: HostBinding, args: [`class.anj-schema-field`,] }],
    _onFieldChange: [{ type: HostListener, args: ['anjChange', ['$event'],] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class SchemaFormService {
    /**
     * \@description Recursively parses a raw JSON schema into an array of parsed fields the UI can work with.
     * @param {?} fieldDefs
     * @return {?}
     */
    static parseSchema(fieldDefs) {
        /** @type {?} */
        const parsedFields = [];
        /** @type {?} */
        const fields = (/** @type {?} */ (fieldDefs.properties));
        for (const propertyName in fields) {
            if (fields[propertyName]) {
                /** @type {?} */
                const prop = fields[propertyName];
                // Make all nested fields required if the parent object is required
                if (prop.type === 'object') {
                    prop.required = (fieldDefs.required.includes(propertyName)) ? Object.keys(prop.properties) : [];
                }
                // Handle for table array
                if (prop.type === 'array' && prop.items && prop.items.type === 'object' && prop.items.properties) {
                    prop.parsedArray = SchemaFormService.parseSchema(prop.items);
                }
                else if (prop.type === 'array') {
                    /** @type {?} */
                    let cloneProp = Object.assign({}, prop);
                    cloneProp = Object.assign(cloneProp, prop.items);
                    prop.parsedList = new ParsedField(propertyName, cloneProp);
                }
                else if (prop.type === 'object' && prop.properties) {
                    prop.parsedFields = SchemaFormService.parseSchema(prop);
                }
                parsedFields.push(new ParsedField(propertyName, prop, fieldDefs.required, fieldDefs.readOnlyOnUI));
            }
        }
        return parsedFields;
    }
    /**
     * \@description Parses a tuple of raw JSON and Data object [SchemaDefinition, SchemaData] into a [ParsedField[], FormGroup].
     * @param {?} payload
     * @return {?}
     */
    static parseSchemaPayload(payload) {
        /** @type {?} */
        const data = payload[1];
        /** @type {?} */
        const fields = SchemaFormService.parseSchema(payload[0]);
        /** @type {?} */
        const group = SchemaFormService.generateFormGroup(fields, data);
        return [fields, group];
    }
    /**
     * \@description Recursively generates a formGroup from a ParsedField[], setting any default data provided.
     * @param {?} fields
     * @param {?} data
     * @return {?}
     */
    static generateFormGroup(fields, data) {
        /** @type {?} */
        const group = new FormGroup({});
        fields.forEach(field => {
            /** @type {?} */
            const initialValue = data && data.hasOwnProperty(field.name) ? data[field.name] : field.default;
            // Handle for table array
            if (Array.isArray(field.parsedArray)) {
                /** @type {?} */
                const newFormArray = new FormArray([], Validators.compose(SchemaFormService.setValidators(field)));
                if (data && data[field.name]) {
                    data[field.name].forEach((dataValue) => {
                        newFormArray.push(SchemaFormService.generateFormGroup(field.parsedArray, dataValue));
                    });
                }
                group.addControl(field.name, newFormArray);
            }
            else if (field.parsedList) { // Handle for list array
                // Handle for list array
                /** @type {?} */
                const newFormArray = new FormArray([], Validators.compose(SchemaFormService.setValidators(field)));
                if (data && data[field.name]) {
                    data[field.name].forEach((dataValue) => {
                        newFormArray.push(new FormControl(dataValue, SchemaFormService.setValidators(field.parsedList)));
                    });
                }
                group.addControl(field.name, newFormArray);
            }
            else if (Array.isArray(field.parsedFields)) { // Handle for nested schema
                // Handle for nested schema
                /** @type {?} */
                const nestedGroup = SchemaFormService.generateFormGroup(field.parsedFields, initialValue);
                group.addControl(field.name, nestedGroup);
            }
            else {
                group.addControl(field.name, new FormControl(initialValue, SchemaFormService.setValidators(field)));
            }
        });
        return group;
    }
    /**
     * \@description Adds the correct validators to each field pending on its type and whether it is required.
     * @param {?} definition
     * @return {?}
     */
    static setValidators(definition) {
        /** @type {?} */
        const validators = [];
        if (definition.required && definition.type === 'boolean') {
            validators.push(AnjunaValidators.requiredCheck);
        }
        else if (definition.required) {
            validators.push(Validators.required);
        }
        if (!isNaN(definition.maximum)) {
            validators.push(Validators.max(definition.maximum));
        }
        if (!isNaN(definition.minimum)) {
            validators.push(Validators.min(definition.minimum));
        }
        if (!isNaN(definition.minLength)) {
            validators.push(Validators.minLength(definition.minLength));
        }
        if (!isNaN(definition.maxLength)) {
            validators.push(Validators.maxLength(definition.maxLength));
        }
        if (definition.format === 'email') {
            validators.push(Validators.email);
        }
        if (definition.pattern) {
            validators.push(Validators.pattern(definition.pattern));
        }
        if (definition.format && definition.format === 'date') {
            validators.push(AnjunaValidators.date);
        }
        if (definition.enforceUnique) {
            validators.push(AnjunaValidators.unique);
        }
        if (!isNaN(definition.minItems) && definition.type === 'array') {
            validators.push(Validators.minLength(definition.minItems));
        }
        return validators;
    }
    /**
     * \@description Parses the keys for generating the table renderer
     * @param {?} field
     * @return {?}
     */
    static parseFieldItemKeys(field) {
        if (field.items && field.items.properties) {
            return Object.keys(field.items.properties);
        }
        return [];
    }
    /**
     * \@description Parses the field names for generating the table renderer.
     * @param {?} field
     * @return {?}
     */
    static parseFieldItemHeaders(field) {
        if (field.items && field.items.properties) {
            return Object.keys(field.items.properties)
                .map(key => (field.items.properties[key].title ? field.items.properties[key].title : key));
        }
        return [];
    }
}
SchemaFormService.decorators = [
    { type: Injectable, args: [{ providedIn: 'root' },] }
];
/** @nocollapse */ SchemaFormService.ngInjectableDef = defineInjectable({ factory: function SchemaFormService_Factory() { return new SchemaFormService(); }, token: SchemaFormService, providedIn: "root" });

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class SchemaForm {
    /**
     * @param {?} schema
     * @param {?} data
     */
    constructor(schema, data) {
        this.fields = {};
        this.fieldNames = [];
        /** @type {?} */
        const payload = SchemaFormService.parseSchemaPayload([schema, data]);
        this.schema = schema;
        this.data = data;
        this.formGroup = payload[1];
        this.formFields = payload[0].map((field) => {
            field.control = this.formGroup.controls[field.name];
            if (field.parsedFields && field.parsedFields.length > 0) {
                this.setFieldParents(field.parsedFields, field);
                this.flattenFields(field.parsedFields, field.control);
            }
            this.fields[field.name] = field;
            return field;
        });
        this.fieldNames = Object.keys(this.fields).filter(name => (this.fields[name].control !== undefined));
    }
    /**
     * @private
     * @param {?} fields
     * @param {?} formGroup
     * @return {?}
     */
    flattenFields(fields, formGroup) {
        fields.forEach((field) => {
            if (field.parsedFields && field.parsedFields.length > 0) {
                this.flattenFields(field.parsedFields, field.control);
            }
            else {
                if (field.type !== 'array') {
                    field.control = this.findFormControl(field);
                }
                else {
                    field.control = ((/** @type {?} */ (formGroup))).controls[field.name];
                }
                this.fields[field.name] = field;
            }
        });
        return fields;
    }
    /**
     * @private
     * @param {?} field
     * @return {?}
     */
    findFormControl(field) {
        /** @type {?} */
        const parentName = field.parent.name;
        /** @type {?} */
        let control;
        /** @type {?} */
        let parent = field.parent;
        while (parent && !control) {
            /** @type {?} */
            const parentControl = parent.control;
            if (parent
                && parentControl
                && ((/** @type {?} */ (parentControl))).controls
                && (parentName in ((/** @type {?} */ (parent.control))).controls)) {
                control = ((/** @type {?} */ (((/** @type {?} */ (parent.control))).controls[parentName]))).controls[field.name];
            }
            else {
                parent = parent.parent;
            }
        }
        return control;
    }
    /**
     * @private
     * @param {?} fields
     * @param {?} parent
     * @return {?}
     */
    setFieldParents(fields, parent) {
        fields.forEach(field => {
            field.parent = parent;
            if (field.parsedFields && field.parsedFields.length > 0) {
                this.setFieldParents(field.parsedFields, field);
            }
        });
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class FieldTemplateDirective {
    /**
     * @param {?} _template
     */
    constructor(_template) {
        this._template = _template;
    }
    /**
     * @return {?}
     */
    get template() {
        return this._template;
    }
}
FieldTemplateDirective.decorators = [
    { type: Directive, args: [{
                selector: '[anjFieldTemplate]'
            },] }
];
/** @nocollapse */
FieldTemplateDirective.ctorParameters = () => [
    { type: TemplateRef }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/* @internal */
/**
 * @abstract
 */
class SchemaControl extends BaseSchema {
    /**
     * @protected
     * @return {?}
     */
    handleChanges() {
        if (this.data && this.schema) {
            this.initGroup();
        }
    }
}
SchemaControl.propDecorators = {
    schema: [{ type: Input }],
    data: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class SchemaFormComponent extends SchemaControl {
    constructor() {
        super();
        this.valueChange = new EventEmitter();
    }
    /**
     * @return {?}
     */
    ngOnChanges() {
        this.handleChanges();
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        if (this.groupSubscription) {
            this.groupSubscription.unsubscribe();
        }
    }
    /**
     * \@description initializes a schema payload imperatively
     * @param {?} schemaForm
     * @return {?}
     */
    initSchemaForm(schemaForm) {
        // Bind them to the component
        this.schemaPayload = schemaForm;
        if (this.schemaPayload) {
            this.groupSubscription = this.schemaPayload.formGroup.valueChanges.subscribe({
                next: (value) => {
                    this.valueChange.emit(value);
                }
            });
        }
    }
    /**
     * @protected
     * @return {?}
     */
    initGroup() {
        this.initSchemaForm(new SchemaForm(this.schema, this.data));
    }
}
SchemaFormComponent.decorators = [
    { type: Component, args: [{
                selector: 'anj-schema-form',
                template: "<ng-container *ngIf=\"schemaPayload && fieldTemplate\">\n    <ng-template [ngTemplateOutlet]=\"fieldTemplate?.template\" [ngTemplateOutletContext]=\"{$implicit: schemaPayload.fields, form: schemaPayload.formGroup, fieldNames: schemaPayload.fieldNames}\"></ng-template>\n</ng-container>\n<ng-content select=\"anj-button-group\"></ng-content>\n",
                changeDetection: ChangeDetectionStrategy.OnPush
            }] }
];
/** @nocollapse */
SchemaFormComponent.ctorParameters = () => [];
SchemaFormComponent.propDecorators = {
    schemaPayload: [{ type: Input }],
    valueChange: [{ type: Output }],
    fieldTemplate: [{ type: ContentChild, args: [FieldTemplateDirective,] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/* @internal */
/**
 * @abstract
 */
class SchemaGroup extends BaseSchema {
    /**
     * @return {?}
     */
    get groupClass() { return this._groupClass || ''; }
    /**
     * @param {?} value
     * @return {?}
     */
    set groupClass(value) {
        this._groupClass = value;
    }
    /**
     * @return {?}
     */
    get fieldClass() { return this._fieldClass || ''; }
    /**
     * @param {?} value
     * @return {?}
     */
    set fieldClass(value) {
        this._fieldClass = value;
    }
    /**
     * @protected
     * @return {?}
     */
    handleChanges() {
        if (this.group) {
            this.initGroup();
        }
    }
}
SchemaGroup.propDecorators = {
    groupClass: [{ type: Input }],
    fieldClass: [{ type: Input }],
    group: [{ type: Input }],
    _fieldClass: [{ type: HostBinding, args: ['attr.fieldClass',] }],
    _groupClass: [{ type: HostBinding, args: ['attr.groupClass',] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class SchemaGroupComponent extends SchemaGroup {
    constructor() {
        super();
        // Define the form group when initialized so that it is defined when needed as a view child property
        this.formGroup = new FormGroup({});
        this.valueChange = new EventEmitter();
    }
    /**
     * @return {?}
     */
    get classList() {
        return `anj-schema-group ${this.groupClass}`;
    }
    /**
     * @return {?}
     */
    ngOnChanges() {
        this.handleChanges();
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        if (this.groupSubscription) {
            this.groupSubscription.unsubscribe();
        }
    }
    /**
     * \@description initializes a schema group imperatively
     * @param {?} __0
     * @return {?}
     */
    initFormGroup([fields, group]) {
        // Bind them to the component
        this.parsedFields = fields;
        this.formGroup = group;
        this.groupSubscription = this.formGroup.valueChanges.subscribe({
            next: (value) => {
                this.valueChange.emit(value);
            }
        });
    }
    /**
     * @protected
     * @return {?}
     */
    initGroup() {
        this.initFormGroup(SchemaFormService.parseSchemaPayload(this.group));
    }
}
SchemaGroupComponent.decorators = [
    { type: Component, args: [{
                selector: 'anj-schema-group',
                template: "<ng-container *ngIf=\"parsedFields\" [formGroup]=\"formGroup\">\n    <anj-schema-field *ngFor=\"let field of parsedFields; trackBy: trackByFn;\"\n                      [field]=\"field\"\n                      [control]=\"formGroup.controls[field.name]\"\n                      [unitOfMeasurement]=\"unitOfMeasurement\"\n                      class=\"{{fieldClass}}\"\n                      [hideLabels]=\"hideLabels\"\n    ></anj-schema-field>\n</ng-container>\n",
                changeDetection: ChangeDetectionStrategy.OnPush,
                encapsulation: ViewEncapsulation.None
            }] }
];
/** @nocollapse */
SchemaGroupComponent.ctorParameters = () => [];
SchemaGroupComponent.propDecorators = {
    valueChange: [{ type: Output }],
    classList: [{ type: HostBinding, args: ['class',] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class SchemaNestedFieldComponent extends SchemaField {
    constructor() {
        super();
    }
    /**
     * @return {?}
     */
    get label() {
        return this._label || '';
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set label(value) {
        this._label = value;
    }
    /**
     * @param {?} newValue
     * @return {?}
     */
    updateNestedField(newValue) {
        this.control.patchValue(newValue);
    }
}
SchemaNestedFieldComponent.decorators = [
    { type: Component, args: [{
                selector: 'anj-schema-nested-field',
                template: "<anj-form-group>\n    <anj-label *ngIf=\"!hideLabels\" [fixed]=\"true\">{{label}}</anj-label>\n</anj-form-group>\n<div class=\"anj-nested\">\n    <ng-container [formGroup]=\"control\">\n        <anj-schema-field *ngFor=\"let nestedField of field.parsedFields; trackBy: trackByFn;\"\n                          [field]=\"nestedField\"\n                          [control]=\"control['controls'][nestedField.name]\"\n        ></anj-schema-field>\n    </ng-container>\n</div>\n\n",
                changeDetection: ChangeDetectionStrategy.OnPush,
                encapsulation: ViewEncapsulation.None,
                styles: ["anj-schema-nested-field .anj-nested{margin:0 0 1rem 1rem}anj-schema-nested-field .anj-form-group.anj-has-value{margin:0}"]
            }] }
];
/** @nocollapse */
SchemaNestedFieldComponent.ctorParameters = () => [];
SchemaNestedFieldComponent.propDecorators = {
    control: [{ type: Input }],
    label: [{ type: Input }],
    isField: [{ type: HostBinding, args: ['class.anj-schema-nested-field',] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/* @internal */
/**
 * @abstract
 */
class SchemaArray extends SchemaField {
    /**
     * @protected
     */
    constructor() {
        super();
    }
    /**
     * @return {?}
     */
    get controlArrayValue() {
        return this.control.value;
    }
    /**
     * @return {?}
     */
    get formArray() {
        return (/** @type {?} */ (this.control));
    }
    /**
     * @param {?=} additionalRows
     * @param {?=} insertPosition
     * @return {?}
     */
    addRow(additionalRows = 1, insertPosition) {
        for (let i = 0; i < additionalRows; i++) {
            /** @type {?} */
            const insertIndex = insertPosition !== undefined ? insertPosition : this.controlArrayValue.length;
            this.formArray.insert(insertIndex, this.newItem());
        }
        this.acknowledgeChange();
    }
    /**
     * @param {?} index
     * @return {?}
     */
    removeRow(index) {
        this.formArray.removeAt(index);
        this.acknowledgeChange();
    }
    /**
     * @return {?}
     */
    atMaxItems() {
        return this.field.maxItems && (this.control.length === this.field.maxItems);
    }
    /**
     * @return {?}
     */
    atMinItems() {
        return this.field.minItems && (this.control.length === this.field.minItems);
    }
    /**
     * @param {?} event
     * @param {?} popper
     * @param {?} content
     * @return {?}
     */
    showToolTip(event, popper, content) {
        clearTimeout(this.timer);
        this.content = content;
        popper.display((/** @type {?} */ (event.target)));
    }
    /**
     * @param {?} popper
     * @return {?}
     */
    hideToolTip(popper) {
        this.timer = window.setTimeout(() => {
            popper.hide();
        }, 750);
    }
    /**
     * @protected
     * @return {?}
     */
    setArray() {
        // If minItems and data is array shorter than minItems, add more empty rows
        if (this.field.minItems &&
            (Array.isArray(this.controlArrayValue) && this.controlArrayValue.length < this.field.minItems)) {
            /** @type {?} */
            const additionalRows = this.field.minItems - this.controlArrayValue.length;
            this.addRow(additionalRows);
            // If Data is not an array, add minimum rows (minItems or 1)
        }
        else if (!Array.isArray(this.controlArrayValue)) {
            /** @type {?} */
            const additionalRows = (this.field.minItems > 0) ? this.field.minItems : 1;
            this.addRow(additionalRows);
        }
    }
}
SchemaArray.propDecorators = {
    control: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * \@experimental
 */
class SchemaListComponent extends SchemaArray {
    /**
     * @param {?} changeRef
     */
    constructor(changeRef) {
        super();
        this.changeRef = changeRef;
    }
    /**
     * @return {?}
     */
    get label() { return this._label || ''; }
    /**
     * @param {?} value
     * @return {?}
     */
    set label(value) {
        this._label = value;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.parsedField = this.field.parsedList;
        this.setArray();
    }
    /**
     * @protected
     * @return {?}
     */
    newItem() {
        return new FormControl();
    }
    /**
     * @protected
     * @return {?}
     */
    acknowledgeChange() {
        this.changeRef.markForCheck();
    }
}
SchemaListComponent.decorators = [
    { type: Component, args: [{
                selector: 'anj-schema-list',
                template: "<anj-form-group class=\"anj-schema-list-label\">\n    <anj-label *ngIf=\"!hideLabels\" [fixed]=\"true\">{{label}}</anj-label>\n</anj-form-group>\n<div class=\"list-container\">\n    <ng-container [formGroup]=\"control\">\n        <div class=\"list-item\" *ngFor=\"let listControl of control.controls; let i = index;\">\n            <ng-container [ngSwitch]=\"parsedField.toInputType()\">\n                <ng-template ngSwitchCase=\"select\">\n                    <anj-form-group>\n                        <anj-select\n                            [formControl]=\"listControl\">\n                            <anj-option *ngFor=\"let option of parsedField.options\" [value]=\"option\">{{option}}</anj-option>\n                            <ng-template [ngIf]=\"listControl.errors\">\n                                <anj-form-error>Please select an option.</anj-form-error>\n                            </ng-template>\n                        </anj-select>\n                    </anj-form-group>\n                </ng-template>\n                <ng-template ngSwitchCase=\"number\">\n                    <anj-form-group>\n                        <anj-input [formControl]=\"listControl\" type=\"number\">\n                            <anj-number-stepper></anj-number-stepper>\n                        </anj-input>\n                        <ng-template [ngIf]=\"listControl.errors\">\n                            <anj-form-error *ngIf=\"listControl.errors?.required\">\n                                {{parsedField.label || parsedField.name}} {{'is required.' | anjSchemaTranslate}}\n                            </anj-form-error>\n                            <anj-form-error *ngIf=\"listControl.errors?.min\">\n                                {{parsedField.label || parsedField.name}} {{'must be at least' | anjSchemaTranslate}} {{listControl.errors.min.min}}.\n                            </anj-form-error>\n                            <anj-form-error *ngIf=\"listControl.errors?.max\">\n                                {{parsedField.label || parsedField.name}} {{'must not be greater than' | anjSchemaTranslate}} {{listControl.errors.max.max}}.\n                            </anj-form-error>\n                        </ng-template>\n                    </anj-form-group>\n                </ng-template>\n                <ng-template ngSwitchDefault>\n                    <anj-form-group>\n                        <anj-input [formControl]=\"listControl\"></anj-input>\n                        <ng-template [ngIf]=\"listControl.errors\">\n                            <anj-form-error *ngIf=\"listControl.errors.required\">\n                                {{parsedField.label || parsedField.name}} {{'is required.' | anjSchemaTranslate}}\n                            </anj-form-error>\n                            <anj-form-error *ngIf=\"listControl.errors.minlength\">\n                                {{parsedField.label || parsedField.name}} {{'must be at least' | anjSchemaTranslate}} {{listControl.errors.minlength.requiredLength}} {{'characters.' | anjSchemaTranslate}}\n                            </anj-form-error>\n                            <anj-form-error *ngIf=\"listControl.errors.maxlength\">\n                                {{parsedField.label || parsedField.name}} {{'must not be greater than' | anjSchemaTranslate}} {{listControl.errors.maxlength.requiredLength}} {{'characters.' | anjSchemaTranslate}}\n                            </anj-form-error>\n                            <anj-form-error *ngIf=\"listControl.errors.pattern\">\n                                {{parsedField.label || parsedField.name}} {{'must adhere to a specific pattern' | anjSchemaTranslate}}{{parsedField.description ? ': ' + parsedField.description : null}}\n                            </anj-form-error>\n                            <anj-form-error *ngIf=\"listControl.errors.email\">\n                                {{parsedField.label || parsedField.name}} {{'must adhere to an email pattern (ex. name@domain.com)' | anjSchemaTranslate}}\n                            </anj-form-error>\n                        </ng-template>\n                    </anj-form-group>\n                </ng-template>\n            </ng-container>\n            <span>\n                <anj-button context=\"icon\" [disabled]=\"atMaxItems()\" (click)=\"addRow(1, i + 1)\"\n                    (mouseenter)=\"showToolTip($event, popper, 'Add an item below')\"\n                    (mouseleave)=\"hideToolTip(popper)\"\n                >\n                    <svg slot=\"icon-only\" xmlns=\"http://www.w3.org/2000/svg\" width=\"24\" height=\"24\" viewBox=\"0 0 24 24\">\n                        <path d=\"M13 7h-2v4H7v2h4v4h2v-4h4v-2h-4V7zm-1-5C6.48 2 2 6.48 2 12s4.48 10 10 10 10-4.48 10-10S17.52 2 12 2zm0 18c-4.41 0-8-3.59-8-8s3.59-8 8-8 8 3.59 8 8-3.59 8-8 8z\"/>\n                    </svg>\n                </anj-button>\n                <anj-button *ngIf=\"i > 0\" context=\"icon\" [disabled]=\"atMinItems()\" (click)=\"removeRow(i)\"\n                    (mouseenter)=\"showToolTip($event, popper, 'Remove this item')\"\n                    (mouseleave)=\"hideToolTip(popper)\"\n                >\n                    <svg slot=\"icon-only\" xmlns=\"http://www.w3.org/2000/svg\" width=\"24\" height=\"24\" viewBox=\"0 0 24 24\">\n                        <path d=\"M6 19c0 1.1.9 2 2 2h8c1.1 0 2-.9 2-2V7H6v12zM8 9h8v10H8V9zm7.5-5l-1-1h-5l-1 1H5v2h14V4z\"/>\n                        <path fill=\"none\" d=\"M0 0h24v24H0V0z\"/>\n                    </svg>\n                </anj-button>\n            </span>\n        </div>\n    </ng-container>\n</div>\n<anj-popover #popper [placement]=\"'bottom'\">\n    {{content  | anjSchemaTranslate}}\n</anj-popover>\n",
                changeDetection: ChangeDetectionStrategy.OnPush,
                encapsulation: ViewEncapsulation.None,
                styles: ["anj-schema-list .anj-schema-list-label{margin-top:1rem}anj-schema-list .list-item{display:flex}anj-schema-list .list-item .form-group{flex:1}"]
            }] }
];
/** @nocollapse */
SchemaListComponent.ctorParameters = () => [
    { type: ChangeDetectorRef }
];
SchemaListComponent.propDecorators = {
    label: [{ type: Input }],
    isField: [{ type: HostBinding, args: ['class.anj-schema-list',] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ReadFieldComponent extends SchemaField {
    /**
     * @param {?} element
     * @param {?} renderer
     */
    constructor(element, renderer) {
        super();
        this.element = element;
        this.renderer = renderer;
    }
    /**
     * @return {?}
     */
    get fieldItemPropertyHeaders() {
        return SchemaFormService.parseFieldItemHeaders(this.field);
    }
    /**
     * @return {?}
     */
    get fieldItemPropertiesKeys() {
        return SchemaFormService.parseFieldItemKeys(this.field);
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        this.renderer.addClass(this.element.nativeElement, this.field.name.split(' ').join('_'));
    }
    /**
     * @return {?}
     */
    toDisplayValue() {
        /** @type {?} */
        let displayValue = this.value ? this.value : '';
        if (Array.isArray(this.value)) {
            displayValue = this.value.join(', ');
        }
        return displayValue;
    }
}
ReadFieldComponent.decorators = [
    { type: Component, args: [{
                selector: 'anj-read-field',
                template: "<ng-container [ngSwitch]=\"field.toInputType()\">\n    <span *ngIf=\"!hideLabels\" class=\"anj-schema-label\">\n        {{field.label}}\n    </span>\n    <span class=\"anj-schema-display-value\" [ngClass]=\"{'nested': field.toInputType() === 'nested'}\">\n        <ng-template ngSwitchCase=\"date\">\n            {{value | date: 'shortDate'}}\n        </ng-template>\n        <ng-template ngSwitchCase=\"checkbox\">\n            <svg *ngIf=\"value\" xmlns=\"http://www.w3.org/2000/svg\" width=\"18\" height=\"18\" viewBox=\"0 0 18 18\">\n                <path d=\"M6.61 11.89L3.5 8.78 2.44 9.84 6.61 14l8.95-8.95L14.5 4z\"/>\n            </svg>\n        </ng-template>\n        <ng-template ngSwitchCase=\"table\">\n            <anj-table [dataSource]=\"value\">\n                <anj-column *ngFor=\"let key of fieldItemPropertyHeaders;\" [field]=\"key\"></anj-column>\n            </anj-table>\n        </ng-template>\n        <ng-template ngSwitchCase=\"nested\">\n            <anj-schema-read\n                [group]=\"[field.nestedDef, value]\"\n            ></anj-schema-read>\n        </ng-template>\n        <ng-template ngSwitchDefault>\n            {{toDisplayValue()}}\n        </ng-template>\n    </span>\n</ng-container>\n",
                changeDetection: ChangeDetectionStrategy.OnPush,
                encapsulation: ViewEncapsulation.None,
                styles: [".anj-read-field{position:relative}.anj-read-field .nested{margin-left:1rem}.anj-read-field .anj-schema-label{color:var(--anj-neutral);font-size:.75rem;letter-spacing:-.0125rem;line-height:1;margin-bottom:0;position:relative;top:0}.anj-read-field .anj-schema-display-value{display:block;min-height:1.75rem}"]
            }] }
];
/** @nocollapse */
ReadFieldComponent.ctorParameters = () => [
    { type: ElementRef },
    { type: Renderer2 }
];
ReadFieldComponent.propDecorators = {
    value: [{ type: Input }],
    isField: [{ type: HostBinding, args: ['class.anj-read-field',] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class SchemaReadComponent extends SchemaGroup {
    constructor() {
        super();
    }
    /**
     * @return {?}
     */
    get classList() {
        return `anj-schema-read ${this.groupClass}`;
    }
    /**
     * @return {?}
     */
    ngOnChanges() {
        this.handleChanges();
    }
    /**
     * @protected
     * @return {?}
     */
    initGroup() {
        this.parsedFields = SchemaFormService.parseSchema(this.group[0]);
        this.values = this.group[1] || {};
    }
}
SchemaReadComponent.decorators = [
    { type: Component, args: [{
                selector: 'anj-schema-read',
                template: "<ng-container *ngIf=\"parsedFields\">\n    <anj-read-field *ngFor=\"let field of parsedFields\"\n        [field]=\"field\"\n        [value]=\"values[field.name]\"\n        class=\"{{fieldClass}}\"\n        [hideLabels]=\"hideLabels\"\n    ></anj-read-field>\n</ng-container>\n",
                changeDetection: ChangeDetectionStrategy.OnPush,
                encapsulation: ViewEncapsulation.None
            }] }
];
/** @nocollapse */
SchemaReadComponent.ctorParameters = () => [];
SchemaReadComponent.propDecorators = {
    classList: [{ type: HostBinding, args: ['class',] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class SchemaTableComponent extends SchemaArray {
    /**
     * @param {?} changeRef
     */
    constructor(changeRef) {
        super();
        this.changeRef = changeRef;
    }
    /**
     * @return {?}
     */
    get label() {
        return this._label || '';
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set label(value) {
        this._label = value;
    }
    // Listen to rows added on anj-table, modify the NG Control respectively
    /**
     * @param {?} event
     * @return {?}
     */
    _onRowAdd(event) {
        this.addRow(1, event.detail.index);
    }
    // Listen to rows removed on anj-table, modify the NG Control respectively
    /**
     * @param {?} event
     * @return {?}
     */
    _onRowRemove(event) {
        this.removeRow(event.detail.index);
    }
    /**
     * @return {?}
     */
    get fieldItemPropertyHeaders() {
        return SchemaFormService.parseFieldItemHeaders(this.field);
    }
    /**
     * @return {?}
     */
    get fieldItemPropertiesKeys() {
        return SchemaFormService.parseFieldItemKeys(this.field);
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.localControlValue = this.controlArrayValue;
        this.setArray();
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        this.acknowledgeChange();
    }
    /**
     * @param {?} def
     * @return {?}
     */
    getDataType(def) {
        // TODO: we probably need to handle at least numbers too
        if (def.type === 'boolean') {
            return 'boolean';
        }
    }
    /**
     * @param {?} fields
     * @param {?} c
     * @return {?}
     */
    findIndex(fields, c) {
        return fields.findIndex(t => t.name === c.field);
    }
    /**
     * @protected
     * @return {?}
     */
    newItem() {
        return SchemaFormService.generateFormGroup(this.field.parsedArray, {});
    }
    /**
     * @protected
     * @return {?}
     */
    acknowledgeChange() {
        this.localControlValue = this.controlArrayValue.slice(0);
        this.changeRef.markForCheck();
    }
}
SchemaTableComponent.decorators = [
    { type: Component, args: [{
                selector: 'anj-schema-table',
                template: "<anj-form-group class=\"anj-schema-table-label\">\n    <anj-label *ngIf=\"!hideLabels\" [fixed]=\"true\">{{label}}</anj-label>\n</anj-form-group>\n<anj-table [dataSource]=\"localControlValue\" column-width=\"auto-fix\" height=\"400px\">\n    <anj-editing\n        mode=\"batch\"\n        creating=\"true\"\n        deleting=\"true\"\n        saving=\"false\"\n        [deleteDisabled]=\"atMinItems()\"\n        [createDisabled]=\"atMaxItems()\"\n    ></anj-editing>\n    <anj-column *ngFor=\"let key of fieldItemPropertyHeaders; let index = index;\"\n                [dataType]=\"getDataType(field.parsedArray[index])\"\n                [field]=\"key\"\n                sortable=\"false\"\n                edit-template=\"controlTemplate\"\n    ></anj-column>\n    <ng-container *anjTemplate=\"let row of 'controlTemplate'; let i = index; let c = column;\">\n        <anj-schema-field\n            *ngIf=\"control.controls[i]\"\n            [field]=\"field.parsedArray[findIndex(field.parsedArray, c)]\"\n            [control]=\"control.controls[i]['controls'][c.field]\"\n            [hideLabels]=\"true\">\n        </anj-schema-field>\n    </ng-container>\n</anj-table>\n",
                changeDetection: ChangeDetectionStrategy.OnPush,
                encapsulation: ViewEncapsulation.None,
                styles: ["anj-schema-table .anj-schema-table-label{margin-top:1rem}anj-schema-table anj-schema-field{display:block}"]
            }] }
];
/** @nocollapse */
SchemaTableComponent.ctorParameters = () => [
    { type: ChangeDetectorRef }
];
SchemaTableComponent.propDecorators = {
    label: [{ type: Input }],
    isField: [{ type: HostBinding, args: ['class.anj-schema-table',] }],
    _onRowAdd: [{ type: HostListener, args: ['anjCreateStart', ['$event'],] }],
    _onRowRemove: [{ type: HostListener, args: ['anjDelete', ['$event'],] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class SchemaContextComponent extends BaseSchema {
    constructor() {
        super();
        this.valueChange = new EventEmitter();
    }
    /**
     * @return {?}
     */
    get groupClass() { return this._groupClass || ''; }
    /**
     * @param {?} value
     * @return {?}
     */
    set groupClass(value) {
        this._groupClass = value;
    }
    /**
     * @return {?}
     */
    get fieldClass() { return this._fieldClass || ''; }
    /**
     * @param {?} value
     * @return {?}
     */
    set fieldClass(value) {
        this._fieldClass = value;
    }
    /**
     * @return {?}
     */
    get mode() { return this._mode || 'view'; }
    /**
     * @param {?} value
     * @return {?}
     */
    set mode(value) {
        this._mode = value;
    }
    /**
     * @return {?}
     */
    get classList() {
        return `anj-schema-context anj-schema-context-${this.context}`;
    }
    /**
     * @param {?} event
     * @return {?}
     */
    updateApp(event) {
        this.valueChange.emit(event);
    }
    /**
     * @return {?}
     */
    ngOnChanges() {
        this.schemaPayload = this.combinePayload();
    }
    /**
     * @private
     * @return {?}
     */
    combinePayload() {
        if (this.mode === 'create' && this.schema) {
            return [this.schema, null];
        }
        else if (this.mode === 'view' && this.schema && !this.data) {
            return [this.schema, {}];
        }
        else if (this.schema && this.data) {
            return [this.schema, this.data];
        }
        return null;
    }
}
SchemaContextComponent.decorators = [
    { type: Component, args: [{
                selector: 'anj-schema-context, anj-custom-fields',
                template: "<ng-container *ngIf=\"mode === 'view'; else formFields\">\n    <anj-schema-read\n        [group]=\"schemaPayload\"\n        [groupClass]=\"groupClass\"\n        [fieldClass]=\"fieldClass\"\n        [hideLabels]=\"hideLabels\"\n    ></anj-schema-read>\n</ng-container>\n<ng-template #formFields>\n    <anj-schema-group\n        [group]=\"schemaPayload\"\n        [groupClass]=\"groupClass\"\n        [fieldClass]=\"fieldClass\"\n        [hideLabels]=\"hideLabels\"\n        [unitOfMeasurement]=\"unitOfMeasurement\"\n        (valueChange)=\"updateApp($event)\"\n    ></anj-schema-group>\n</ng-template>\n",
                changeDetection: ChangeDetectionStrategy.OnPush,
                encapsulation: ViewEncapsulation.None
            }] }
];
/** @nocollapse */
SchemaContextComponent.ctorParameters = () => [];
SchemaContextComponent.propDecorators = {
    groupClass: [{ type: Input }],
    fieldClass: [{ type: Input }],
    mode: [{ type: Input }],
    schema: [{ type: Input }],
    data: [{ type: Input }],
    context: [{ type: Input }],
    valueChange: [{ type: Output }],
    schemaGroup: [{ type: ViewChild, args: [SchemaGroupComponent,] }],
    classList: [{ type: HostBinding, args: ['class',] }],
    _fieldClass: [{ type: HostBinding, args: ['attr.fieldClass',] }],
    _groupClass: [{ type: HostBinding, args: ['attr.groupClass',] }],
    _mode: [{ type: HostBinding, args: ['attr.mode',] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const translations = {
    en: {
        'is required.': 'is required.',
        'is not a valid date.': 'is not a valid date.',
        'Please select an option.': 'Please select an option.',
        'must be at least': 'must be at least',
        'must not be greater than': 'must not be greater than',
        'must not have duplicate values.': 'must not have duplicate values.',
        'must have at least': 'must have at least',
        'entries.': 'entries.',
        'characters.': 'characters.',
        'must adhere to a specific pattern': 'must adhere to a specific pattern',
        'must adhere to an email pattern (ex. name@domain.com)': 'must adhere to an email pattern (ex. name@domain.com)',
        'Add an item below': 'Add an item below',
        'Remove this item': 'Remove this item',
        'Add a row below': 'Add a row below',
        'Remove this row': 'Remove this row',
    },
    fr: {
        'is required.': 'est obligatoire.',
        'is not a valid date.': 'n’est pas une date valide.',
        'Please select an option.': 'Sélectionnez une option.',
        'must be at least': 'doit être au moins',
        'must not be greater than': 'ne doit pas dépasser',
        'must not have duplicate values.': 'ne doit pas inclure de valeurs dupliquées.',
        'must have at least': 'doit inclure au moins',
        'entries.': 'entrées.',
        'characters.': 'caractères.',
        'must adhere to a specific pattern': 'doit correspondre à un format spécifique',
        'must adhere to an email pattern (ex. name@domain.com)': 'doit correspondre à un format d’e-mail (par ex., nom@domaine.com)',
        'Add an item below': 'Ajouter un élément ci-dessous',
        'Remove this item': 'Supprimer cet élément',
        'Add a row below': 'Ajouter une ligne ci-dessous',
        'Remove this row': 'Supprimer cette ligne',
    },
    de: {
        'is required.': 'ist erforderlich.',
        'is not a valid date.': 'ist kein gültiges Datum.',
        'Please select an option.': 'Wählen Sie eine Option aus.',
        'must be at least': 'muss mindestens folgenden Wert haben:',
        'must not be greater than': 'darf nicht größer als folgender Wert sein:',
        'must not have duplicate values.': 'darf keine duplizierten Werte aufweisen.',
        'must have at least': 'muss mindestens',
        'entries.': 'Einträge aufweisen.',
        'characters.': 'Zeichen lang sein.',
        'must adhere to a specific pattern': 'muss ein bestimmtes Muster befolgen',
        'must adhere to an email pattern (ex. name@domain.com)': 'muss ein bestimmtes E-Mail-Muster befolgen (Bsp.: name@domain.com)',
        'Add an item below': 'Element unterhalb hinzufügen',
        'Remove this item': 'Dieses Element entfernen',
        'Add a row below': 'Zeile unterhalb hinzufügen',
        'Remove this row': 'Diese Zeile entfernen',
    },
    ja: {
        'is required.': 'は必須です。',
        'is not a valid date.': 'は有効な日付ではありません。',
        'Please select an option.': 'オプションを選択してください。',
        'must be at least': 'の最小長さは',
        'must not be greater than': 'の最大長さは',
        'must not have duplicate values.': 'の値は重複させることはできません。',
        'must have at least': 'には少なくとも',
        'entries.': '個のエントリが必要です。',
        'characters.': '文字にする必要があります。',
        'must adhere to a specific pattern': 'は指定のパターン',
        'must adhere to an email pattern (ex. name@domain.com)': 'は電子メールのパターン (name@domain.com など) に従う必要があります',
        'Add an item below': '以下のアイテムを追加',
        'Remove this item': 'このアイテムを削除',
        'Add a row below': '以下の行を追加',
        'Remove this row': 'この行を削除',
    },
    es: {
        'is required.': 'es obligatoria.',
        'is not a valid date.': 'no es una fecha válida',
        'Please select an option': 'Por favor, seleccione una opción',
        'must be at least': 'debe ser al menos',
        'must not be greater than': 'no debe ser mayor que',
        'must not have duplicate values': 'no debe contener valores duplicados',
        'must have at least': 'debe tener al menos',
        'entries.': 'entradas.',
        'characters.': 'caracteres.',
        'must adhere to a specific pattern': 'debe ajustarse a un patrón específico',
        'must adhere to an email pattern (ex. name@domain.com)': 'debe adherirse a un patrón de correo electrónico (por ejemplo, nombre@dominio.com)',
        'Add an item below': 'Agregar un elemento abajo',
        'Remove this item': 'Eliminar este elemento',
        'Add a row below': 'Agregar una fila abajo',
        'Remove this row': 'Eliminar esta fila'
    }
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * \@internal
 */
class SchemaTranslationPipe extends BaseTranslationPipe {
    constructor() {
        super(...arguments);
        this.translations = translations;
    }
}
SchemaTranslationPipe.decorators = [
    { type: Pipe, args: [{
                name: 'anjSchemaTranslate'
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class SchemaTierComponent extends SchemaTableComponent {
    /**
     * @param {?} changeRef
     */
    constructor(changeRef) {
        super(changeRef);
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        super.ngOnInit();
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        this.acknowledgeChange();
        if (changes.unitOfMeasurement && !changes.unitOfMeasurement.firstChange) {
            this.refreshTiers();
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    _onRowAdd(event) {
        /** @type {?} */
        const length = this.control.length;
        this.addRow(1, length);
    }
    /**
     * @param {?} index
     * @return {?}
     */
    tierToChange(index) {
        /** @type {?} */
        const nextRow = this.control.at(index + 1);
        /** @type {?} */
        const currentRowTierTo = this.control.at(index).get('tierTo').value;
        if (nextRow) {
            nextRow.patchValue({ tierFrom: currentRowTierTo + this.unitOfMeasurement });
        }
    }
    /**
     * @param {?=} additionalRows
     * @param {?=} insertPosition
     * @return {?}
     */
    addRow(additionalRows = 1, insertPosition) {
        super.addRow(additionalRows, insertPosition);
    }
    /**
     * @protected
     * @return {?}
     */
    newItem() {
        /** @type {?} */
        const group = SchemaFormService.generateFormGroup(this.field.parsedArray, {});
        /** @type {?} */
        const previousTier = this.control.at(this.control.length - 1);
        /** @type {?} */
        const previousTierTo = previousTier.get('tierTo').value;
        group.patchValue({ tierFrom: previousTierTo + this.unitOfMeasurement });
        return group;
    }
    /**
     * @private
     * @return {?}
     */
    refreshTiers() {
        this.control.controls.forEach((row, index) => {
            /** @type {?} */
            const tierToValue = row.get('tierFrom').value;
            row.patchValue({ tierFrom: tierToValue + this.unitOfMeasurement });
        });
    }
}
SchemaTierComponent.decorators = [
    { type: Component, args: [{
                selector: 'anj-schema-tier',
                template: "<anj-form-group class=\"anj-schema-table-label\">\n    <anj-label *ngIf=\"!hideLabels\" [fixed]=\"true\">{{label}}</anj-label>\n</anj-form-group>\n<anj-table [dataSource]=\"localControlValue\" column-width=\"auto-fix\" height=\"400px\">\n    <anj-editing\n        mode=\"batch\"\n        creating=\"true\"\n        deleting=\"true\"\n        saving=\"false\"\n        [deleteDisabled]=\"atMinItems()\"\n        [createDisabled]=\"atMaxItems()\"\n    ></anj-editing>\n    <anj-column *ngFor=\"let key of fieldItemPropertyHeaders; let index = index;\"\n                [dataType]=\"getDataType(field.parsedArray[index])\"\n                [field]=\"key\"\n                sortable=\"false\"\n                edit-template=\"controlTemplate\"\n    ></anj-column>\n    <ng-container *anjTemplate=\"let row of 'controlTemplate'; let i = index; let c = column;\">\n        <anj-schema-field\n            *ngIf=\"control.controls[i]\"\n            [field]=\"field.parsedArray[findIndex(field.parsedArray, c)]\"\n            [control]=\"control.controls[i]['controls'][c.field]\"\n            [index]=\"i\"\n            [hideLabels]=\"true\"\n            (tierToChange)=\"tierToChange($event)\">\n        </anj-schema-field>\n    </ng-container>\n</anj-table>\n",
                styles: ["anj-schema-table .anj-schema-table-label{margin-top:1rem}anj-schema-table anj-schema-field{display:block}"]
            }] }
];
/** @nocollapse */
SchemaTierComponent.ctorParameters = () => [
    { type: ChangeDetectorRef }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class AnjunaMetaUIModule {
}
AnjunaMetaUIModule.decorators = [
    { type: NgModule, args: [{
                declarations: [
                    FieldTemplateDirective,
                    SchemaFieldComponent,
                    SchemaFormComponent,
                    SchemaGroupComponent,
                    SchemaNestedFieldComponent,
                    SchemaTranslationPipe,
                    SchemaListComponent,
                    SchemaReadComponent,
                    SchemaTableComponent,
                    ReadFieldComponent,
                    SchemaContextComponent,
                    SchemaTierComponent
                ],
                imports: [
                    CommonModule,
                    ReactiveFormsModule,
                    FormsModule,
                    AnjunaCoreModule
                ],
                exports: [
                    FieldTemplateDirective,
                    SchemaFieldComponent,
                    SchemaFormComponent,
                    SchemaGroupComponent,
                    SchemaNestedFieldComponent,
                    SchemaListComponent,
                    SchemaReadComponent,
                    SchemaTableComponent,
                    ReadFieldComponent,
                    SchemaContextComponent
                ],
                schemas: [CUSTOM_ELEMENTS_SCHEMA]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { SchemaFieldComponent, SchemaFormComponent, SchemaGroupComponent, SchemaNestedFieldComponent, SchemaListComponent, ReadFieldComponent, SchemaReadComponent, SchemaTableComponent, SchemaContextComponent, ParsedField, SchemaForm, SchemaFormService, AnjunaMetaUIModule, BaseSchema as ɵb, SchemaArray as ɵf, SchemaControl as ɵc, SchemaField as ɵa, SchemaGroup as ɵe, FieldTemplateDirective as ɵd, SchemaTranslationPipe as ɵg, SchemaTierComponent as ɵh };

//# sourceMappingURL=anjuna-angular-meta-ui.js.map